# Generated by Django 2.2.9 on 2020-03-19 15:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0020_merge_20191003_0948'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ('order', 'id'), 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.AddField(
            model_name='option',
            name='order',
            field=models.IntegerField(blank=True, null=True, verbose_name='Ordering', help_text="Controls the ordering of product options on product detail pages"),
        ),
    ]
