# Generated by Django 4.2.11 on 2025-04-24 07:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0030_merge_product_code"),
        ("catalogue", "0031_productcategoryhierarchy"),
    ]

    operations = []
