# Generated by Django 3.2.23 on 2024-01-09 15:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0027_attributeoption_code_attributeoptiongroup_code_and_more'),
        ('catalogue', '0027_merge_20220708_1223'),
    ]

    operations = [
    ]
