# Generated by Django 4.2.11 on 2024-12-04 10:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0029_merge_20240911_1251"),
        ("catalogue", "0029_product_code"),
    ]

    operations = []
