# Generated by Django 2.2.13 on 2021-07-27 11:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0024_use_oscar_seo'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ['order', 'name'], 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.RemoveField(
            model_name='category',
            name='seo_description',
        ),
        migrations.RemoveField(
            model_name='category',
            name='seo_keywords',
        ),
        migrations.RemoveField(
            model_name='category',
            name='seo_title',
        ),
        migrations.RemoveField(
            model_name='product',
            name='seo_description',
        ),
        migrations.RemoveField(
            model_name='product',
            name='seo_keywords',
        ),
        migrations.RemoveField(
            model_name='product',
            name='seo_title',
        ),
        migrations.RenameField(
            model_name='category',
            old_name='parents_are_public',
            new_name='ancestors_are_public'
        ),
        migrations.AlterField(
            model_name='category',
            name='ancestors_are_public',
            field=models.BooleanField(db_index=True, default=True, help_text='The ancestors of this category are public', verbose_name='Ancestor categories are public'),
        ),
    ]
