# Generated by Django 2.1.9 on 2019-10-03 09:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0017_category_is_public'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='parents_are_public',
            field=models.BooleanField(default=True, help_text='The parents of this category are public', verbose_name='Parent categories are public'),
        ),
    ]
