# Generated by Django 2.2.13 on 2021-07-20 12:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0021_auto_20201005_0844'),
        ('catalogue', '0022_order_index'),
    ]

    operations = [
    ]
