# Generated by Django 2.0.13 on 2019-07-11 12:27

import django.core.validators
from django.db import migrations, models
from ocyan.plugin.oscar.fender import ALL_TAX_RATES, get_default_tax_rate as vat_default

class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0017_category_is_public'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='seo_description',
            field=models.CharField(blank=True, help_text='SEO Meta descriptions should be less than 300 characters, if left empty the product description will be used', max_length=1024, null=True, verbose_name='SEO Meta description'),
        ),
        migrations.AddField(
            model_name='category',
            name='seo_keywords',
            field=models.CharField(blank=True, help_text="Meta Keywords don't make any improvements in SEO, use Meta description instead.", max_length=1024, null=True, verbose_name='SEO Meta keywords'),
        ),
        migrations.AddField(
            model_name='category',
            name='seo_title',
            field=models.CharField(blank=True, help_text='SEO Meta titles should be less than 60 characters, if left empty the product title will be used', max_length=255, null=True, verbose_name='SEO Meta title'),
        ),
        migrations.AddField(
            model_name='product',
            name='long_description',
            field=models.TextField(blank=True, help_text='A far more elaborate, but optional description of the product', null=True, verbose_name='Content'),
        ),
        migrations.AddField(
            model_name='product',
            name='seo_description',
            field=models.CharField(blank=True, help_text='SEO Meta descriptions should be less than 300 characters, if left empty the product description will be used', max_length=1024, null=True, verbose_name='SEO Meta description'),
        ),
        migrations.AddField(
            model_name='product',
            name='seo_keywords',
            field=models.CharField(blank=True, help_text="Meta Keywords don't make any improvements in SEO, use Meta description instead.", max_length=1024, null=True, verbose_name='SEO Meta keywords'),
        ),
        migrations.AddField(
            model_name='product',
            name='seo_title',
            field=models.CharField(blank=True, help_text='SEO Meta titles should be less than 60 characters, if left empty the product title will be used', max_length=255, null=True, verbose_name='SEO Meta title'),
        ),
        migrations.AddField(
            model_name='product',
            name='vat_rate',
            field=models.CharField(choices=ALL_TAX_RATES, default=vat_default(), max_length=5, null=True, verbose_name='VAT rate'),
        ),
        migrations.AddField(
            model_name='productattributevalue',
            name='value_embed',
            field=models.TextField(blank=True, help_text='This can be found in the URL of a video. So for example: https://www.youtube.com/watch?v=dkIU-ygvtps. In this case {dkIU-ygvtps} is the Youtube Video ID', null=True, verbose_name='Youtube Video ID'),
        ),
        migrations.AlterField(
            model_name='productattribute',
            name='type',
            field=models.CharField(choices=[('text', 'Text'), ('integer', 'Integer'), ('boolean', 'True / False'), ('float', 'Float'), ('richtext', 'Rich Text'), ('date', 'Date'), ('datetime', 'Datetime'), ('option', 'Option'), ('multi_option', 'Multi Option'), ('entity', 'Entity'), ('file', 'File'), ('image', 'Image'), ('embed', 'Embed')], default='text', max_length=20, verbose_name='Type'),
        ),
    ]
