# -*- coding: utf-8 -*-

__all__ = ["OscarCatalogueAppPluginConfig"]


def __getattr__(name):
    if name != "OscarCatalogueAppPluginConfig":
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
    from .entrypoint import (
        OcyanOscar3CatalogueAppPluginConfig as OscarCatalogueAppPluginConfig,
    )

    return OscarCatalogueAppPluginConfig
