# Generated by Django 3.2.13 on 2022-07-08 10:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0025_remove_seo_fields'),
        ('catalogue', '0026_predefined_product_options'),
    ]

    operations = [
    ]
