"""
This file is only used in oxyan with oscar 3
"""

# pylint: disable=no-member
from oscar.apps.catalogue import apps

from .shared.appconfig import CatalogueAppConfigMixin


class CatalogueConfig(CatalogueAppConfigMixin, apps.CatalogueConfig):
    name = "ocyan.plugin.oscar_catalogue.catalogue"

    def get_urls(self):
        review_urlpatterns = apps.CatalogueReviewsOnlyConfig.get_urls(self)
        return self.post_process_urls(
            review_urlpatterns + self.new_urls() + self.old_urls()
        )
