# Generated by Django 5.2.3 on 2025-07-17 07:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0032_category_exclude_from_menu_category_long_description'),
        ('catalogue', '0032_merge_20250424_0937'),
    ]

    operations = [
    ]
