# Generated by Django 4.2.11 on 2024-09-11 10:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0028_merge_in_unique_codes"),
        ("catalogue", "0028_product_priority"),
    ]

    operations = []
