# Generated by Django 2.0.13 on 2019-10-03 09:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0019_product_priority'),
        ('catalogue', '0018_category_parents_are_public'),
    ]

    operations = [
    ]
