<div id="embed">
    <iframe
        class="lazy"
        width="100%" height="100%" 
        data-src="https://www.youtube.com/embed/{{value_embed}}"
        frameborder="0" allowfullscreen="allowfullscreen">
    </iframe>
</div>