# Generated by Django 4.2.15 on 2024-09-11 10:37

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0027_attributeoption_code_attributeoptiongroup_code_and_more"),
    ]

    """
        We empty this operation because 0019_product_priority already did this,
        but we want this 0028 migration for compatibility with migrating older oscar projects to ocyan
    """
    operations = []
