# Generated by Django 2.0.13 on 2019-07-18 10:25

from django.db import migrations, models
import django.core.validators

class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0018_ocyan_additions'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='product',
            options={'ordering': ['-priority', '-date_created'], 'verbose_name': 'Product', 'verbose_name_plural': 'Products'},
        ),
        migrations.AddField(
            model_name='product',
            name='priority',
            field=models.SmallIntegerField(db_index=True, default=0, help_text='The highest priority products are shown first', verbose_name='Priority', validators=[django.core.validators.MinValueValidator(-1)]),
        ),
    ]
