from ocyan.core.fender import config

DEFAULT_SLUG = "product"
MINIMIM_QUANTITY_ATTRIBUTE = config.get(
    "oscar_catalogue", "minimum_quantity_attribute_code", "min_quantity"
)
SLUG_ID_SEPARATOR = config.get("oscar_catalogue", "slug_id_separator", "-")
